//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
// Spelling menu implementation
//****************************************************************************

#include "qwerty.h"
#include "spell.h"
#include "briefedit.h"

// extract word from position inside line
static QString extract_word(QString str) {
	QChar qc = str[0];
	if (!qc.isLetterOrNumber())
		return 0;
	int i = 0;
	while (qc.isLetterOrNumber() || qc == '-') {
		i++;
		qc = str[i];
	}

	str.truncate(i);
	return str;
}







void Editor::advanceStartWord() {
//	LOG("enter advanceStartWord function with index %d", x_);
	int lines = e_->numLines();
//	LOG("lines %d", lines);

	while (1) {
		QChar ch = line_.at(x_);
		for (int i = 0; !ch.isNull(); i++, x_++) {
			ch = line_.at(x_);
			if (ch.isLetterOrNumber() || ch == '<' || ch == '/' || ch == '&') { // check for utf8 also
//				LOG("return advanceStartWord function with index %d", x_);
				return;
			}
		}

		if ((y_ + 1) >= lines) {
//			LOG("advanceStartWord advance end of file detected");
			e_->setCursorPosition(y_, x_, false);
			break;
		}

		y_++;
		x_ = 0;
		line_ = e_->textLine(y_);
		e_->setCursorPosition(y_, x_, false);
//		LOG("advanceStartWord advance para %d, index %d", y_, x_);
	}
}

QString Editor::advanceEndWord() {
	if (e_->atEnd())
		return QString("");

	xword_ = x_;
//	LOG("enter advanceEndWord with index %d", x_);

	QChar ch = line_.at(x_);
	int i;
	for (i = 0; !ch.isNull() && (ch.isLetterOrNumber() || ch =='-'); i++) {
		x_++;
		ch = line_.at(x_);
	}
	if (i == 0) {
//		LOG("return from 0");
		return QString("");
	}


//	LOG("advanceWord selection %d/%d %d/%d", y_, xword_, y_, x_);
	//lucian: it might return all string!!!
	QString retval = line_.mid(xword_, x_ - xword_);
	if (retval.isNull()) {
//		LOG("return isNull");
		return QString("");
	}

//	LOG("advanceEndWord %s", (const char *)retval.utf8());
//	LOG("exit advanceEndWord with index %d", x_);
	return retval;
}


bool Editor::skeepXML() {
	// save the status in case we run into a non-terminated xml tag
	int tmp_x = x_;
	int tmp_y = y_;

//	LOG("enter skeepXML with index %d", x_);
	int lines = e_->numLines();
	LOG("lines %d", lines);
	QChar ch = line_.at(x_);

	if (ch == '<') {
//		LOG("Editor::skeepXML - xml detected");
		while (1) {
			x_++;
			ch = line_.at(x_);
			if (ch == '>')
				break;
			if (ch.isNull()) {
				// advance here to a new line
				if ((y_ + 1) > lines) {
					x_ = tmp_x + 1;
					y_ = tmp_y;
					line_ = e_->textLine(y_);
//					LOG("advanceXML end of file detected");
					e_->setCursorPosition(y_, x_, false);
					return true;
				}
				y_++;
				x_ = 0;
				line_ = e_->textLine(y_);
//				LOG("skeepXML sml advance para %d, index %d", y_, x_);
			}
		}

//		LOG("exit skeepXML with index %d", x_);
		return true;
	} else if (ch == '/' || ch == '&') {
//		LOG("Editor::skeepXML - path detected");
		x_++;
		QChar ch = line_.at(x_);
		while (ch.isLetterOrNumber()) {
			x_++;
			ch = line_.at(x_);
		}
//		LOG("exit skeepXML path with index %d", x_);
		return true;
	}


//	LOG("exit skeepXML with index %d", x_);
	return false;

}

void Editor::spellAdvance()
{
	if (e_->atEnd()) {
		e_->selectAll(true);
		msgScreen("end of file");
		return;
	}

//	e_->moveCursor(QTextEdit::MoveForward, true);
//	e_->moveCursor(QTextEdit::MoveBackward, true);

	e_->getCursorPosition(&y_, &x_);
	line_ = e_->textLine(y_);

	// just in case the cursor is in the middle of a word
	advanceEndWord();

	if (pdict_ == 0)
		initDictionary();

	DbDict *hptr = pdict_->getMaster();
	DbUser *uptr = pdict_->getUser();
	DbUser *iptr = pdict_->getIgnore();
	DbUser *fptr = pdict_->getFixed();
	while (1) {
		e_->selectAll(false);
		// advance to a new word
		advanceStartWord();
		if (e_->atEnd()) {
			cleanMWords();
			msgScreen("end of file");
			return;
		}

		// skeep XML markup
		if (skeepXML())
			continue;

		//e->cursorWordForward(true);
		QString text = advanceEndWord();
// hasnumeric added here
		if (text.isEmpty() || text.isNull() || hasnumeric(text))
			continue;

		QString extracted = extract_word(text);

		Q3CString utext = extracted.utf8();
		const char *ctext = utext;
		char *active_word = (char *) ctext;

		if (active_word == 0) {
			LOG("couldn't extract word");
			// lucian to fix1: we might get here because the word started with an
			// utf8 char that is not a valid word char
			// just jump over this char for now
			e_->moveCursor(Q3TextEdit::MoveForward, false);
			continue;
		}

		LOG("text active:%s", active_word);
		if (!hptr->findString(active_word) &&	// if not in dictionary
		    !fptr->findString(active_word, true) &&	 // if not in fixed/names dictionary
		    !uptr->findString(active_word) &&	// if not in user dictionary
		    !iptr->findString(active_word)) {	// if not in ignore dictionary
			Stats::instance()->notFound();
			cleanMWords();
			// skeep lookalike for words consisting in a single letter
			if (strlen(active_word) == 1) {
				break;
			}
			doLookalike(active_word);
			break;
		} else
			Stats::instance()->found();
	}

	e_->setCursorPosition(y_, x_, false);
	e_->setSelection(y_, xword_, y_, x_);
}


void Editor::cleanMWords() {
	mwords_->changeItem(mw1_, QString(" "));
	mwords_->changeItem(mw2_, QString(" "));
	mwords_->changeItem(mw3_, QString(" "));
	mwords_->changeItem(mw4_, QString(" "));
	mwords_->changeItem(mw5_, QString(" "));
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
}

void Editor::replaceWord1() {
	if (*word1_ == ' ' || *word1_ == '\0')
		return;
	LOG("replaceWord1 %s", word1_);
	char word[100];
	strcpy(word, word1_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord2() {
	if (*word2_ == ' ' || *word2_ == '\0')
		return;
	LOG("replaceWord2 %s", word2_);
	char word[100];
	strcpy(word, word2_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord3() {
	if (*word3_ == ' ' || *word3_ == '\0')
		return;
	LOG("replaceWord3 %s", word3_);
	char word[100];
	strcpy(word, word3_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord4() {
	if (*word4_ == ' ' || *word4_ == '\0')
		return;
	LOG("replaceWord4 %s", word4_);
	char word[100];
	strcpy(word, word4_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord5() {
	if (*word5_ == ' ' || *word5_ == '\0')
		return;
	LOG("replaceWord5 %s", word5_);
	char word[100];
	strcpy(word, word5_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::spellIgnore()
{
	QString text = e_->markedText();
	QString extracted = extract_word(text);
	if (extracted.isEmpty()) {
		text = text.mid(1);
		extracted = extract_word(text);
	}
	if (extracted.isEmpty()) {
		spellAdvance();
		return;
	}
	Q3CString utext = extracted.utf8();
	const char *ctext = utext;
	char *active_word = (char *) ctext;
	pdict_->getIgnore()->setString(active_word);
	spellAdvance();
}

void Editor::spellAdd()
{
	QString text = e_->markedText();
	QString extracted = extract_word(text);
	if (extracted.isEmpty()) {
		text = text.mid(1);
		extracted = extract_word(text);
	}
	if (extracted.isEmpty()) {
		spellAdvance();
		return;
	}
	Q3CString utext = extracted.utf8();
	const char *ctext = utext;
	char *active_word = (char *) ctext;
	pdict_->getUser()->setString(active_word);
	spellAdvance();
}

void Editor::spellName()
{
	QString text = e_->markedText();
	QString extracted = extract_word(text);
	if (extracted.isEmpty()) {
		text = text.mid(1);
		extracted = extract_word(text);
	}
	if (extracted.isEmpty()) {
		spellAdvance();
		return;
	}
	Q3CString utext = extracted.utf8();
	const char *ctext = utext;
	char *active_word = (char *) ctext;
	pdict_->getFixed()->setString(active_word, true);
	spellAdvance();
}

void Editor::insertLookalike(char *str) {
	ASSERT(str != 0);

	if (*word1_ == '\0') {
		strcpy(word1_, str);
		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
		return;
	}
	if (*word2_ == '\0') {
		strcpy(word2_, str);
		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
		return;
	}
	if (*word3_ == '\0') {
		strcpy(word3_, str);
		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
		return;
	}
	if (*word4_ == '\0') {
		strcpy(word4_, str);
		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
		return;
	}
	if (*word5_ == '\0') {
		strcpy(word5_, str);
		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
		return;
	}
}


void Editor::doLookalike(char *active_word) {
	if (pdict_ == 0)
		initDictionary();

	DbDict *hptr = pdict_->getMaster();

	// grab any valid words form dict based on active_words
	hptr->findLookalike(active_word, word1_, word2_, word3_, word4_, word5_);
	mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
	mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
	mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
	mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
	mwords_->changeItem(mw5_, QString::fromUtf8(word5_));

	// look for words with i to â rule
	char tmp_word[100];
	bool look_done = lookalike1(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike1");
		insertLookalike(tmp_word);
	}

	// look for the second apparition in words with i to â rule
	look_done = lookalike2(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike2");
		insertLookalike(tmp_word);
     }

	// look for the first apparition in words with î to â rule
	look_done = lookalike3(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike3");
		insertLookalike(tmp_word);
     }

	// flip char lookalike
	look_done = lookalike4(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike4");
		insertLookalike(tmp_word);
	}

	// fix teutzz spelling: tz -> ţ
	look_done = lookalike5(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike5");
		insertLookalike(tmp_word);
	}

	// fix sh spelling: sh -> ş
	look_done = lookalike6(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike5");
		insertLookalike(tmp_word);
	}

	// if necessary change from cedilla in commabelow
	if (Global::instance()->lang_ro_commabelow) {
		char *w = UTF8::strcedilla2commabelow(word1_);
		strcpy(word1_, w);
		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word2_);
		strcpy(word2_, w);
		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word3_);
		strcpy(word3_, w);
		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word4_);
		strcpy(word4_, w);
		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word5_);
		strcpy(word5_, w);
		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
		delete [] w;
	}

	LOG("word1:%s", word1_);
	LOG("word2:%s", word2_);
	LOG("word3:%s", word3_);
	LOG("word4:%s", word4_);
	LOG("word5:%s", word5_);
}

// flip char lookalike
//lucian: does not suport flipping romanian utf chars yet
bool Editor::lookalike4(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char newword[100 + 1];
	strncpy(newword, active_word, 100);
	unsigned char *ptr = (unsigned char *) newword;
	if (pdict_ == 0)
		initDictionary();
	DbDict *hptr = pdict_->getMaster();

	while (*ptr != 0 && *(ptr + 1) != 0) {
		if (!(*ptr & 0x80  || *(ptr + 1) & 0x80)) {

			char tmp = *ptr;
			*ptr = *(ptr + 1);
			*(ptr + 1) = tmp;
			if (hptr->findString(newword)) {
				strcpy(dest, newword);
				return true;
			}

			tmp = *ptr;
			*ptr = *(ptr + 1);
			*(ptr + 1) = tmp;
		}
		if (*ptr & 0x80)
			ptr += 2;
		else
			ptr++;
	}
	return false;
}

// first i to â lookalike
bool Editor::lookalike1(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "i")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "i");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "â");
		strcat(ptrtmp, ptr + 1);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		if (pdict_ == 0)
			initDictionary();
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

// second i to â lookalike
bool Editor::lookalike2(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "i")) != 0) {
		if ((ptr = strstr(ptr + 1, "i")) != 0) {
			strcpy(dest, active_word);
			ptr = strstr(dest, "i");
			ASSERT(ptr != 0);
			if (ptr == 0)
				return false; // just in case
			ptr = strstr(ptr + 1, "i");
			ASSERT(ptr != 0);
			if (ptr == 0)
				return false; // just in case
			char ptrtmp[100];
			strcpy(ptrtmp, "â");
			strcat(ptrtmp, ptr + 1);
			strcpy(ptr, ptrtmp);
			// look for the new word in dictionary
			if (pdict_ == 0)
				initDictionary();
			DbDict *hptr = pdict_->getMaster();
			if (hptr->findString(dest))
				return true;
			else
				*dest = '\0';
		}
	}
	return false;
}

// first î to â lookalike
bool Editor::lookalike3(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "î")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "î");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "â");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		if (pdict_ == 0)
			initDictionary();
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

bool Editor::lookalike5(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "tz")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "tz");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "ţ");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		if (pdict_ == 0)
			initDictionary();
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

bool Editor::lookalike6(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "sh")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "sh");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "ş");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		if (pdict_ == 0)
			initDictionary();
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

